#!/bin/sh
VERSION=v4
CPU=440fp
TARGET=powerpc-${CPU}-linux-gnu
LINUX_VERSION=3.7.3
BUSYBOX_VERSION=1.20.2
DTS=virtex440-unisim
INITRD_SIZE=65536 # max 65536 KB or reconfigure the linux kernel
NUM_PROCESSORS=$(cat /proc/cpuinfo | cut -f 1 | grep vendor_id | wc -l)
HERE=$(pwd)
MY_DIR=$(cd $(dirname $0); pwd)
KERNELDIR=${MY_DIR}/linux-${LINUX_VERSION}

TARBALLS_DIR=${MY_DIR}/downloads
BOOT_ROM=boot.elf
INITRD_IMG=initrd.img
DEVICE_TREE_BLOB=device-tree
VMLINUX=vmlinux
INITRD_DIR=${MY_DIR}/initrd
SCRIPTS_DIR=${MY_DIR}/scripts

Usage ()
{
	echo "Usage: $0 <command> [<path-to-prebuilt-miBench-dir>] [<path-to-cross-compiler>]"
	echo "  command set:"
	echo "     all          build everything"
	echo "     clean        clean everything"
	echo "     dist-bin     tar all binaries and data sets in a unique tarball"
	echo "     dist-source  tar all sources in a unique tarball"
	echo "  path-to-precompiled-miBench-dir:"
	echo "     Path to directory containing prebuilt MiBench version 1 benchmarks"
	echo "     (e.g. \${HOME}/mibench-v1-bin-\${TARGET}-v1)"
	echo "  path-to-cross-compiler:"
	echo "     Path to directory of gcc C/C++ cross-compiler for ${TARGET}"
	echo "     either as argument or into environment variable POWERPC_CROSS_COMPILER_PREFIX"
	echo "     (e.g. /opt/crosstool/gcc-4.4.6-glibc-2.9/${TARGET})"
	echo "     Use crosstool-0.43 or preferably crosstool-ng to get a working cross compiler."
}

CMD="$1"
case "${CMD}" in
	all)
		MIBENCH_BUILD_DIR="$2"
		if ! [ -d "${MIBENCH_BUILD_DIR}" ] || ! [ -d "${MIBENCH_BUILD_DIR}/automotive" ]; then
			echo "FATAL: Can't find prebuilt MiBench version 1 benchmarks"
			Usage
			exit 255
		fi
		if [ -z "${POWERPC_CROSS_COMPILER_PREFIX}" ]; then
			if [ -z "$3" ]; then
				echo "FATAL: Please specify path to cross-compiler"
				Usage
				exit 255
			fi
			POWERPC_CROSS_COMPILER_PREFIX="$3"
		fi
		;;
	clean)
		;;
	dist-bin)
		FILE_LIST="\
		boot.elf \
		vmlinux \
		device-tree-6m.dtb \
		device-tree-8m.dtb \
		device-tree-12m.dtb \
		device-tree-16m.dtb \
		device-tree-24m.dtb \
		device-tree-32m.dtb \
		device-tree-40m.dtb \
		device-tree-64m.dtb \
		device-tree-96m.dtb \
		initrd_automotive_basicmath_large.img \
		initrd_automotive_basicmath_small.img \
		initrd_automotive_bitcount_large.img \
		initrd_automotive_bitcount_small.img \
		initrd_automotive_qsort_large.img \
		initrd_automotive_qsort_small.img \
		initrd_automotive_susan_large.img \
		initrd_automotive_susan_small.img \
		initrd_consumer_jpeg_large.img \
		initrd_consumer_jpeg_small.img \
		initrd_consumer_lame_large.img \
		initrd_consumer_lame_small.img \
		initrd_consumer_typeset_large.img \
		initrd_consumer_typeset_small.img \
		initrd.img \
		initrd_network_dijkstra_large.img \
		initrd_network_dijkstra_small.img \
		initrd_network_patricia_large.img \
		initrd_network_patricia_small.img \
		initrd_office_stringsearch_large.img \
		initrd_office_stringsearch_small.img \
		initrd_security_blowfish_large.img \
		initrd_security_blowfish_small.img \
		initrd_security_pgp.img \
		initrd_security_rijndael_large.img \
		initrd_security_rijndael_small.img \
		initrd_security_sha_large.img \
		initrd_security_sha_small.img \
		initrd_telecomm_adpcm_large.img \
		initrd_telecomm_adpcm_small.img \
		initrd_telecomm_crc32_large.img \
		initrd_telecomm_crc32_small.img \
		initrd_telecomm_fft_large.img \
		initrd_telecomm_fft_small.img \
		initrd_telecomm_gsm_large.img \
		initrd_telecomm_gsm_small.img \
		busybox-${BUSYBOX_VERSION}/busybox_unstripped \
		config/sim_config_automotive_basicmath_large.xml \
		config/sim_config_automotive_basicmath_small.xml \
		config/sim_config_automotive_bitcount_large.xml \
		config/sim_config_automotive_bitcount_small.xml \
		config/sim_config_automotive_qsort_large.xml \
		config/sim_config_automotive_qsort_small.xml \
		config/sim_config_automotive_susan_large.xml \
		config/sim_config_automotive_susan_small.xml \
		config/sim_config_consumer_jpeg_large.xml \
		config/sim_config_consumer_jpeg_small.xml \
		config/sim_config_consumer_lame_large.xml \
		config/sim_config_consumer_lame_small.xml \
		config/sim_config_consumer_typeset_large.xml \
		config/sim_config_consumer_typeset_small.xml \
		config/sim_config_network_dijkstra_large.xml \
		config/sim_config_network_dijkstra_small.xml \
		config/sim_config_network_patricia_large.xml \
		config/sim_config_network_patricia_small.xml \
		config/sim_config_office_stringsearch_large.xml \
		config/sim_config_office_stringsearch_small.xml \
		config/sim_config_security_blowfish_large.xml \
		config/sim_config_security_blowfish_small.xml \
		config/sim_config_security_pgp.xml \
		config/sim_config_security_rijndael_large.xml \
		config/sim_config_security_rijndael_small.xml \
		config/sim_config_security_sha_large.xml \
		config/sim_config_security_sha_small.xml \
		config/sim_config_telecomm_adpcm_large.xml \
		config/sim_config_telecomm_adpcm_small.xml \
		config/sim_config_telecomm_crc32_large.xml \
		config/sim_config_telecomm_crc32_small.xml \
		config/sim_config_telecomm_fft_large.xml \
		config/sim_config_telecomm_fft_small.xml \
		config/sim_config_telecomm_gsm_large.xml \
		config/sim_config_telecomm_gsm_small.xml \
		config/sim_config.xml \
		config/automotive_basicmath_large.debug.mac \
		config/automotive_basicmath_small.debug.mac \
		config/automotive_bitcount_large.debug.mac \
		config/automotive_bitcount_small.debug.mac \
		config/automotive_qsort_large.debug.mac \
		config/automotive_qsort_small.debug.mac \
		config/automotive_susan_large.debug.mac \
		config/automotive_susan_small.debug.mac \
		config/consumer_jpeg_large.debug.mac \
		config/consumer_jpeg_small.debug.mac \
		config/consumer_lame_large.debug.mac \
		config/consumer_lame_small.debug.mac \
		config/consumer_typeset_large.debug.mac \
		config/consumer_typeset_small.debug.mac \
		config/debug.mac \
		config/network_dijkstra_large.debug.mac \
		config/network_dijkstra_small.debug.mac \
		config/network_patricia_large.debug.mac \
		config/network_patricia_small.debug.mac \
		config/office_stringsearch_large.debug.mac \
		config/office_stringsearch_small.debug.mac \
		config/security_blowfish_large.debug.mac \
		config/security_blowfish_small.debug.mac \
		config/security_pgp.debug.mac \
		config/security_rijndael_large.debug.mac \
		config/security_rijndael_small.debug.mac \
		config/security_sha_large.debug.mac \
		config/security_sha_small.debug.mac \
		config/telecomm_adpcm_large.debug.mac \
		config/telecomm_adpcm_small.debug.mac \
		config/telecomm_crc32_large.debug.mac \
		config/telecomm_crc32_small.debug.mac \
		config/telecomm_fft_large.debug.mac \
		config/telecomm_fft_small.debug.mac \
		config/telecomm_gsm_large.debug.mac \
		config/telecomm_gsm_small.debug.mac \
		check.sh \
		output_automotive_basicmath_large.ref.txt \
		output_automotive_basicmath_small.ref.txt \
		output_automotive_bitcount_large.ref.txt \
		output_automotive_bitcount_small.ref.txt \
		output_automotive_qsort_large.ref.txt \
		output_automotive_qsort_small.ref.txt \
		output_automotive_susan_large.ref.txt \
		output_automotive_susan_small.ref.txt \
		output_consumer_jpeg_large.ref.txt \
		output_consumer_jpeg_small.ref.txt \
		output_consumer_lame_large.ref.txt \
		output_consumer_lame_small.ref.txt \
		output_consumer_typeset_large.ref.txt \
		output_consumer_typeset_small.ref.txt \
		output_network_dijkstra_large.ref.txt \
		output_network_dijkstra_small.ref.txt \
		output_network_patricia_large.ref.txt \
		output_network_patricia_small.ref.txt \
		output_office_stringsearch_large.ref.txt \
		output_office_stringsearch_small.ref.txt \
		output_security_blowfish_large.ref.txt \
		output_security_blowfish_small.ref.txt \
		output_security_pgp.ref.txt \
		output_security_rijndael_large.ref.txt \
		output_security_rijndael_small.ref.txt \
		output_security_sha_large.ref.txt \
		output_security_sha_small.ref.txt \
		output_telecomm_adpcm_large.ref.txt \
		output_telecomm_adpcm_small.ref.txt \
		output_telecomm_crc32_large.ref.txt \
		output_telecomm_crc32_small.ref.txt \
		output_telecomm_fft_large.ref.txt \
		output_telecomm_fft_small.ref.txt \
		output_telecomm_gsm_large.ref.txt \
		output_telecomm_gsm_small.ref.txt \
		"

		PKG_NAME="unisim-virtex5fxt-linux-distro-bin-${VERSION}"

		rm -rf ${HERE}/${PKG_NAME}
		rm -f ${HERE}/${PKG_NAME}.tar.gz
		mkdir -p ${HERE}/${PKG_NAME}

		for FILE in ${FILE_LIST}; do
			mkdir -p ${HERE}/${PKG_NAME}/$(dirname ${FILE})
			cp ${MY_DIR}/${FILE} ${HERE}/${PKG_NAME}/${FILE} || exit -1
		done

		cat << EOF > "${HERE}/${PKG_NAME}/README"
This package provides a mini Linux distribution based on Busybox and the MiBench version 1 benchmarks.
The package contains:
- prebuilt boot (boot.elf)
- a prebuilt Linux kernel image (vmlinux)
- prebuilt device tree images (*.dtb)
- configuration files for the UNISIM Virtex 5 FXT Simulator (1.0beta9+) (config/sim_config*.xml)
- prebuilt initial RAM disk images (*.img) that themselves contain:
    - busybox
    - prebuilt binaries of MiBench version 1 (http://www.eecs.umich.edu/mibench) for target ${TARGET}
    - the original MiBench version 1 datasets and scripts to run each benchmark on the native target
- the reference outputs of MiBench benchmarks for PPC440FP (sim_out*.ref.txt).
- a script (check.sh) to run regression tests for the initial RAM disk builds and the UNISIM Virtex 5 FXT simulator.

Note: consumer/mad, consumer/tiff-v3.5.4 and office/sphinx are not included because we were not able to build them
EOF
		cd ${HERE}
		tar zcvf ${HERE}/${PKG_NAME}.tar.gz ${PKG_NAME}
		rm -rf ${HERE}/${PKG_NAME}
		;;
	dist-source)
		FILE_LIST="\
		boot/boot.lds \
		boot/Makefile \
		boot/ppc44x_regs.h \
		boot/boot.S \
		downloads/busybox-${BUSYBOX_VERSION}.tar.bz2 \
		downloads/linux-${LINUX_VERSION}.tar.bz2 \
		config/busybox-${BUSYBOX_VERSION}-config \
		config/linux-${LINUX_VERSION}-config \
		config/virtex440-unisim-6m.dts \
		config/virtex440-unisim-8m.dts \
		config/virtex440-unisim-12m.dts \
		config/virtex440-unisim-16m.dts \
		config/virtex440-unisim-24m.dts \
		config/virtex440-unisim-32m.dts \
		config/virtex440-unisim-40m.dts \
		config/virtex440-unisim-64m.dts \
		config/virtex440-unisim-96m.dts \
		patches/patch-cttyhack-${BUSYBOX_VERSION} \
		patches/patch-linux-${LINUX_VERSION}-virtex5fxt-fpu \
		scripts/mkcompile_h \
		scripts/init_automotive_basicmath_large.sh \
		scripts/init_automotive_basicmath_small.sh \
		scripts/init_automotive_bitcount_large.sh \
		scripts/init_automotive_bitcount_small.sh \
		scripts/init_automotive_qsort_large.sh \
		scripts/init_automotive_qsort_small.sh \
		scripts/init_automotive_susan_large.sh \
		scripts/init_automotive_susan_small.sh \
		scripts/init_consumer_jpeg_large.sh \
		scripts/init_consumer_jpeg_small.sh \
		scripts/init_consumer_lame_large.sh \
		scripts/init_consumer_lame_small.sh \
		scripts/init_consumer_typeset_large.sh \
		scripts/init_consumer_typeset_small.sh \
		scripts/init_network_dijkstra_large.sh \
		scripts/init_network_dijkstra_small.sh \
		scripts/init_network_patricia_large.sh \
		scripts/init_network_patricia_small.sh \
		scripts/init_office_stringsearch_large.sh \
		scripts/init_office_stringsearch_small.sh \
		scripts/init_security_blowfish_large.sh \
		scripts/init_security_blowfish_small.sh \
		scripts/init_security_pgp.sh \
		scripts/init_security_rijndael_large.sh \
		scripts/init_security_rijndael_small.sh \
		scripts/init_security_sha_large.sh \
		scripts/init_security_sha_small.sh \
		scripts/init.sh \
		scripts/inittab \
		scripts/init_telecomm_adpcm_large.sh \
		scripts/init_telecomm_adpcm_small.sh \
		scripts/init_telecomm_crc32_large.sh \
		scripts/init_telecomm_crc32_small.sh \
		scripts/init_telecomm_fft_large.sh \
		scripts/init_telecomm_fft_small.sh \
		scripts/init_telecomm_gsm_large.sh \
		scripts/init_telecomm_gsm_small.sh \
		scripts/rcS \
		config/sim_config_automotive_basicmath_large.xml \
		config/sim_config_automotive_basicmath_small.xml \
		config/sim_config_automotive_bitcount_large.xml \
		config/sim_config_automotive_bitcount_small.xml \
		config/sim_config_automotive_qsort_large.xml \
		config/sim_config_automotive_qsort_small.xml \
		config/sim_config_automotive_susan_large.xml \
		config/sim_config_automotive_susan_small.xml \
		config/sim_config_consumer_jpeg_large.xml \
		config/sim_config_consumer_jpeg_small.xml \
		config/sim_config_consumer_lame_large.xml \
		config/sim_config_consumer_lame_small.xml \
		config/sim_config_consumer_typeset_large.xml \
		config/sim_config_consumer_typeset_small.xml \
		config/sim_config_network_dijkstra_large.xml \
		config/sim_config_network_dijkstra_small.xml \
		config/sim_config_network_patricia_large.xml \
		config/sim_config_network_patricia_small.xml \
		config/sim_config_office_stringsearch_large.xml \
		config/sim_config_office_stringsearch_small.xml \
		config/sim_config_security_blowfish_large.xml \
		config/sim_config_security_blowfish_small.xml \
		config/sim_config_security_pgp.xml \
		config/sim_config_security_rijndael_large.xml \
		config/sim_config_security_rijndael_small.xml \
		config/sim_config_security_sha_large.xml \
		config/sim_config_security_sha_small.xml \
		config/sim_config_telecomm_adpcm_large.xml \
		config/sim_config_telecomm_adpcm_small.xml \
		config/sim_config_telecomm_crc32_large.xml \
		config/sim_config_telecomm_crc32_small.xml \
		config/sim_config_telecomm_fft_large.xml \
		config/sim_config_telecomm_fft_small.xml \
		config/sim_config_telecomm_gsm_large.xml \
		config/sim_config_telecomm_gsm_small.xml \
		config/sim_config.xml \
		config/automotive_basicmath_large.debug.mac \
		config/automotive_basicmath_small.debug.mac \
		config/automotive_bitcount_large.debug.mac \
		config/automotive_bitcount_small.debug.mac \
		config/automotive_qsort_large.debug.mac \
		config/automotive_qsort_small.debug.mac \
		config/automotive_susan_large.debug.mac \
		config/automotive_susan_small.debug.mac \
		config/consumer_jpeg_large.debug.mac \
		config/consumer_jpeg_small.debug.mac \
		config/consumer_lame_large.debug.mac \
		config/consumer_lame_small.debug.mac \
		config/consumer_typeset_large.debug.mac \
		config/consumer_typeset_small.debug.mac \
		config/debug.mac \
		config/network_dijkstra_large.debug.mac \
		config/network_dijkstra_small.debug.mac \
		config/network_patricia_large.debug.mac \
		config/network_patricia_small.debug.mac \
		config/office_stringsearch_large.debug.mac \
		config/office_stringsearch_small.debug.mac \
		config/security_blowfish_large.debug.mac \
		config/security_blowfish_small.debug.mac \
		config/security_pgp.debug.mac \
		config/security_rijndael_large.debug.mac \
		config/security_rijndael_small.debug.mac \
		config/security_sha_large.debug.mac \
		config/security_sha_small.debug.mac \
		config/telecomm_adpcm_large.debug.mac \
		config/telecomm_adpcm_small.debug.mac \
		config/telecomm_crc32_large.debug.mac \
		config/telecomm_crc32_small.debug.mac \
		config/telecomm_fft_large.debug.mac \
		config/telecomm_fft_small.debug.mac \
		config/telecomm_gsm_large.debug.mac \
		config/telecomm_gsm_small.debug.mac \
		check.sh \
		output_automotive_basicmath_large.ref.txt \
		output_automotive_basicmath_small.ref.txt \
		output_automotive_bitcount_large.ref.txt \
		output_automotive_bitcount_small.ref.txt \
		output_automotive_qsort_large.ref.txt \
		output_automotive_qsort_small.ref.txt \
		output_automotive_susan_large.ref.txt \
		output_automotive_susan_small.ref.txt \
		output_consumer_jpeg_large.ref.txt \
		output_consumer_jpeg_small.ref.txt \
		output_consumer_lame_large.ref.txt \
		output_consumer_lame_small.ref.txt \
		output_consumer_typeset_large.ref.txt \
		output_consumer_typeset_small.ref.txt \
		output_network_dijkstra_large.ref.txt \
		output_network_dijkstra_small.ref.txt \
		output_network_patricia_large.ref.txt \
		output_network_patricia_small.ref.txt \
		output_office_stringsearch_large.ref.txt \
		output_office_stringsearch_small.ref.txt \
		output_security_blowfish_large.ref.txt \
		output_security_blowfish_small.ref.txt \
		output_security_pgp.ref.txt \
		output_security_rijndael_large.ref.txt \
		output_security_rijndael_small.ref.txt \
		output_security_sha_large.ref.txt \
		output_security_sha_small.ref.txt \
		output_telecomm_adpcm_large.ref.txt \
		output_telecomm_adpcm_small.ref.txt \
		output_telecomm_crc32_large.ref.txt \
		output_telecomm_crc32_small.ref.txt \
		output_telecomm_fft_large.ref.txt \
		output_telecomm_fft_small.ref.txt \
		output_telecomm_gsm_large.ref.txt \
		output_telecomm_gsm_small.ref.txt \
		$(basename $0) \
		"

		PKG_NAME="unisim-virtex5fxt-linux-distro-source-${VERSION}"

		rm -rf ${HERE}/${PKG_NAME}
		rm -f ${HERE}/${PKG_NAME}.tar.gz
		mkdir -p ${HERE}/${PKG_NAME}

		for FILE in ${FILE_LIST}; do
			mkdir -p ${HERE}/${PKG_NAME}/$(dirname ${FILE})
			cp ${MY_DIR}/${FILE} ${HERE}/${PKG_NAME}/${FILE} || exit -1
		done

		cat << EOF > "${HERE}/${PKG_NAME}/README"
This package provides the source code of a mini Linux distribution based on Busybox and the MiBench version 1 benchmarks.

Requirements:
- A Linux host machine with a decent installation of development packages
- sudo
- being in the list of sudoers (people who can run sudo)

This package contains:
- the source of the boot program for the UNISIM Virtex 5 FXT Simulator (1.0beta8+)
- the original Linux kernel source tarball (downloads/linux*.tar.gz)
- the original Busybox source tarball (downloads/busybox*.tar.gz)
- the device trees for the UNISIM Virtex 5 FXT Simulator (1.0beta8+) (config/*.dts)
- the patches to make the Linux kernel and Busybox run on the UNISIM Virtex 5 FXT Simulator (1.0beta8+)
- the init scripts of initial RAM disks (scripts/*)
- the configuration files for building the Linux kernel and Busybox
- the configuration files for the UNISIM Virtex 5 FXT Simulator (1.0beta9+) (config/sim_config*.xml)
- the reference outputs of MiBench benchmarks for PPC440FP (sim_out*.ref.txt).
- a script (check.sh) to run regression tests for the initial RAM disk builds and the UNISIM Virtex 5 FXT simulator.
EOF
		cd ${HERE}
		tar zcvf ${HERE}/${PKG_NAME}.tar.gz ${PKG_NAME}
		rm -rf ${HERE}/${PKG_NAME}
		;;
	*)
		echo "FATAL: Please specify a command"
		Usage
		exit 255
		;;
esac

mkdir -p ${TARBALLS_DIR}

################################################################################
#                                BOOT ROM                                      #
################################################################################

case "${CMD}" in
	all)
		echo "Building boot ROM (${BOOT_ROM}). Please wait..."
		make -C boot all AS=${POWERPC_CROSS_COMPILER_PREFIX}/bin/${TARGET}-as LD=${POWERPC_CROSS_COMPILER_PREFIX}/bin/${TARGET}-ld CPP=${POWERPC_CROSS_COMPILER_PREFIX}/bin/${TARGET}-cpp
		if [ $? -ne 0 ]; then
			echo "Building boot ROM failed"
			exit -1
		fi
		cp -f ${MY_DIR}/boot/boot.elf ${MY_DIR}/${BOOT_ROM}
		;;
	clean)
		echo "Cleaning boot ROM (${BOOT_ROM}). Please wait..."
		make -C boot clean
		rm -f "${MY_DIR}/${BOOT_ROM}"
		;;
esac


################################################################################
#                               LINUX KERNEL                                   #
################################################################################

LINUX_CONFIG=${MY_DIR}/config/linux-${LINUX_VERSION}-config
MKCOMPILE_H=${MY_DIR}/scripts/mkcompile_h
PATCH_LINUX_VIRTEX5FXT_FPU=${MY_DIR}/patches/patch-linux-${LINUX_VERSION}-virtex5fxt-fpu

case "${CMD}" in
	all)
		if [ ! -e ${TARBALLS_DIR}/linux-${LINUX_VERSION}.tar.bz2 ]; then
			echo "Downloading Linux kernel source tarball. Please wait..."
			wget ftp://ftp.free.fr/mirrors/ftp.kernel.org/linux/kernel/v$(printf "${LINUX_VERSION}" | cut -f 1-2 -d .)/linux-${LINUX_VERSION}.tar.bz2 -P ${TARBALLS_DIR}
			if [ $? -ne 0 ]; then
				echo "Downloading Linux kernel source tarball failed"
				exit -1
			fi
		fi

		if [ ! -d linux-${LINUX_VERSION} ]; then
			echo "Uncompressing Linux kernel source tarball. Please wait..."
			tar jxvf ${TARBALLS_DIR}/linux-${LINUX_VERSION}.tar.bz2
			cd ${MY_DIR}/linux-${LINUX_VERSION}
			patch -p1 < ${PATCH_LINUX_VIRTEX5FXT_FPU} || exit -1
			cp -f ${LINUX_CONFIG} ${MY_DIR}/linux-${LINUX_VERSION}/.config
			cp -f ${MKCOMPILE_H} ${MY_DIR}/linux-${LINUX_VERSION}/scripts/mkcompile_h
			cd ${MY_DIR}
		fi

		echo "Building Linux kernel (vmlinux). Please wait..."
		make -j ${NUM_PROCESSORS} -C linux-${LINUX_VERSION} ARCH=powerpc CROSS_COMPILE=${POWERPC_CROSS_COMPILER_PREFIX}/bin/${TARGET}- V=1 vmlinux
		if [ $? -ne 0 ]; then
			echo "Building Linux kernel (vmlinux) failed"
			exit -1
		fi
		cp -f "${MY_DIR}/linux-${LINUX_VERSION}/vmlinux" "${MY_DIR}/${VMLINUX}"
		echo "Building device tree blob. Please wait..."
		for SZ in 6m 8m 12m 16m 24m 32m 40m 64m 96m; do
			cp -f "${MY_DIR}/config/${DTS}-${SZ}.dts" "${MY_DIR}/linux-${LINUX_VERSION}/arch/powerpc/boot/dts/${DTS}-${SZ}.dts"
			make -j ${NUM_PROCESSORS} -C linux-${LINUX_VERSION} ARCH=powerpc CROSS_COMPILE=${POWERPC_CROSS_COMPILER_PREFIX}/bin/${TARGET}- V=1 "${DTS}-${SZ}.dtb"
			if [ $? -ne 0 ]; then
				echo "Building device tree blob (${DTS}-${SZ}.dtb) failed"
				exit -1
			fi
			cp -f "${MY_DIR}/linux-${LINUX_VERSION}/arch/powerpc/boot/${DTS}-${SZ}.dtb" "${MY_DIR}/${DEVICE_TREE_BLOB}-${SZ}.dtb"
		done
		;;
	clean)
		echo "Cleaning Linux kernel (vmlinux). Please wait..."
		rm -rf ${MY_DIR}/linux-${LINUX_VERSION}
		rm -f vmlinux
		echo "Cleaning Device tree blob (${DTS}.dtb). Please wait..."
		for SZ in 6m 8m 12m 16m 24m 32m 40m 64m 96m; do
			rm -f "${MY_DIR}/${DEVICE_TREE_BLOB}-${SZ}"
		done
		;;
esac

################################################################################
#                                    BUSYBOX                                   #
################################################################################

case "${CMD}" in
	all)
		if [ ! -e ${TARBALLS_DIR}/busybox-${BUSYBOX_VERSION}.tar.bz2 ]; then
			echo "Downloading Busybox source tarball. Please wait..."
			wget --random-wait  --user-agent="Mozilla 5.0" http://www.busybox.net/downloads/busybox-${BUSYBOX_VERSION}.tar.bz2 -P ${TARBALLS_DIR}
			if [ $? -ne 0 ]; then
				echo "Downloading Busybox source tarball failed"
				exit -1
			fi
		fi

		BUSYBOX_CONFIG=${MY_DIR}/config/busybox-${BUSYBOX_VERSION}-config
		BUSYBOX_PATCH_CTTYHACK=${MY_DIR}/patches/patch-cttyhack-${BUSYBOX_VERSION}
		BUSYBOX_PATCH_LIBBB=${MY_DIR}/patches/patch-libbb-${BUSYBOX_VERSION}

		if [ ! -d busybox-${BUSYBOX_VERSION} ]; then
			echo "Uncompressing Busybox source tarball. Please wait..."
			tar jxvf ${TARBALLS_DIR}/busybox-${BUSYBOX_VERSION}.tar.bz2

			if [ -e "${BUSYBOX_PATCH_CTTYHACK}" ]; then
				echo "Patching cttyhack. Please wait..."
				cd ${MY_DIR}/busybox-${BUSYBOX_VERSION}
				patch -p1 < ${BUSYBOX_PATCH_CTTYHACK} || exit -1
			fi
			if [ -e "${BUSYBOX_PATCH_LIBBB}" ]; then
				echo "Patching libbb. Please wait..."
				cd ${MY_DIR}/busybox-${BUSYBOX_VERSION}
				patch -p1 < ${BUSYBOX_PATCH_LIBBB} || exit -1
			fi
			
			cd ${MY_DIR}
		fi

		echo "Building Busybox. Please wait..."
		cat ${BUSYBOX_CONFIG} | sed -e "s#\(CONFIG_CROSS_COMPILER_PREFIX=\).*\$#\1\"${POWERPC_CROSS_COMPILER_PREFIX}/bin/${TARGET}-\"#" -e "s#\(CONFIG_PREFIX=\).*\$#\1\"${INITRD_DIR}\"#" > busybox-${BUSYBOX_VERSION}/.config
		make -j ${NUM_PROCESSORS} -C busybox-${BUSYBOX_VERSION} busybox V=1
		if [ $? -ne 0 ]; then
			echo "Building Busybox failed"
			exit -1
		fi
		;;
	clean)
		echo "Cleaning Busybox. Please wait..."
		rm -rf busybox-${BUSYBOX_VERSION}
		;;
esac

################################################################################
#                                    INITRD                                    #
################################################################################

MakeInitRD ()
{
	local INITRD_IMG="$1"
	local INITRD_SIZE="$2"
	local INIT_SCRIPT="$3"
	local EXTRA_DIR="$4"
	local EXTRA_FILES="$5"

	echo "Creating uncompressed initial ramdisk image of ${INITRD_SIZE} KB"
	dd if=/dev/zero of="${INITRD_IMG}" count="${INITRD_SIZE}" bs=1024 || exit 255

	echo "Creating ext2 file system in initial ramdisk image (only sudoers can do that)"
	sudo mke2fs -F -m 0 "${INITRD_IMG}" || exit 255

	echo "Mounting uncompressed initial ramdisk image (only sudoers can do that)"
	sudo rm -rf "${INITRD_DIR}" || exit 255
	mkdir -p ${INITRD_DIR} || exit 255
	sudo mount -o loop "${INITRD_IMG}" "${INITRD_DIR}" || exit 255

	echo "Creating initial ramdisk directory structure... (only sudoers can do that)"
	sudo mkdir -p "${INITRD_DIR}/mnt" || exit 255
	sudo mkdir -p "${INITRD_DIR}/sbin" || exit 255
	sudo mkdir -p "${INITRD_DIR}/etc" || exit 255
	sudo mkdir -p "${INITRD_DIR}/etc/init.d" || exit 255
	sudo mkdir -p "${INITRD_DIR}/usr/bin" || exit 255
	sudo mkdir -p "${INITRD_DIR}/usr" || exit 255
	sudo mkdir -p "${INITRD_DIR}/usr/share" || exit 255
	sudo mkdir -p "${INITRD_DIR}/bin" || exit 255
	sudo mkdir -p "${INITRD_DIR}/sys" || exit 255
	sudo mkdir -p "${INITRD_DIR}/dev" || exit 255
	sudo mkdir -p "${INITRD_DIR}/proc" || exit 255
	sudo mkdir -p "${INITRD_DIR}/lib" || exit 255
	sudo mkdir -p "${INITRD_DIR}/lib/modules" || exit 255
	sudo mkdir -p "${INITRD_DIR}/lib/modules/${LINUX_VERSION}" || exit 255
	sudo mkdir -p "${INITRD_DIR}/lib/modules/${LINUX_VERSION}/kernel" || exit 255
	sudo mkdir -p "${INITRD_DIR}/lib/modules/${LINUX_VERSION}/kernel/drivers" || exit 255
	sudo mkdir -p "${INITRD_DIR}/lib/modules/${LINUX_VERSION}/kernel/drivers/misc" || exit 255
	sudo mkdir -p "${INITRD_DIR}/tmp" || exit 255
	sudo mkdir -p "${INITRD_DIR}/opt" || exit 255

	echo "Installing initialization scripts... (only sudoers can do that)"
	sudo cp "${SCRIPTS_DIR}/rcS" "${INITRD_DIR}/etc/init.d" || exit 255
	sudo cp "${SCRIPTS_DIR}/inittab" "${INITRD_DIR}/etc" || exit 255
	if [ -e "${SCRIPTS_DIR}/${INIT_SCRIPT}" ]; then
		sudo cp "${SCRIPTS_DIR}/${INIT_SCRIPT}" "${INITRD_DIR}/etc/init.sh" || exit 255
	else
		sudo cp "${SCRIPTS_DIR}/init.sh" "${INITRD_DIR}/etc/init.sh" || exit 255
	fi

	echo "Installing busybox into initial ramdisk... (only sudoers can do that)"
	sudo make -C "busybox-${BUSYBOX_VERSION}" install || exit 255
	if [ $? -ne 0 ]; then
		echo "Installing Busybox failed"
		exit -1
	fi

	echo "Installing shared libraries into initial ramdisk...(only sudoers can do that)"
	for SO_FILE in libstdc++.so.6 libm.so.6 libgcc_s.so.1 libc.so.6 ld.so.1 libpthread.so.0 librt.so.1; do
		if [ -e "${POWERPC_CROSS_COMPILER_PREFIX}/powerpc-${CPU}-linux-gnu/sysroot/lib/${SO_FILE}" ]; then
			sudo ${POWERPC_CROSS_COMPILER_PREFIX}/bin/powerpc-${CPU}-linux-gnu-strip \
				--strip-unneeded -o "${INITRD_DIR}/lib/${SO_FILE}" \
				"${POWERPC_CROSS_COMPILER_PREFIX}/powerpc-${CPU}-linux-gnu/sysroot/lib/${SO_FILE}" || exit 255
		elif [ -e "${POWERPC_CROSS_COMPILER_PREFIX}/powerpc-${CPU}-linux-gnu/lib/${SO_FILE}" ]; then
			sudo ${POWERPC_CROSS_COMPILER_PREFIX}/bin/powerpc-${CPU}-linux-gnu-strip \
				--strip-unneeded -o "${INITRD_DIR}/lib/${SO_FILE}" \
				"${POWERPC_CROSS_COMPILER_PREFIX}/powerpc-${CPU}-linux-gnu/lib/${SO_FILE}" || exit 255
		else
			echo "${SO_FILE} not found"
			exit 255
		fi

		sudo chmod +x "${INITRD_DIR}/lib/${SO_FILE}" || exit 255
	done

	cd "${INITRD_DIR}/lib"
	sudo ln -s libstdc++.so.6 libstdc++.so || exit 255
	sudo ln -s libm.so.6 libm.so || exit 255
	sudo ln -s libgcc_s.so.1 libgcc_s.so || exit 255
	sudo ln -s libc.so.6 libc.so || exit 255
	sudo ln -s librt.so.1 librt.so || exit 255
	sudo ln -s libpthread.so.0 libpthread.so || exit 255
	cd "${MY_DIR}"

	echo "Making device nodes in /dev (only sudoers can do that)"
	sudo mknod "${INITRD_DIR}/dev/fb0" c 29 0 || exit 255
	sudo mknod "${INITRD_DIR}/dev/psaux" c 10 1 || exit 255
	sudo mknod "${INITRD_DIR}/dev/console" c 5 1 || exit 255
	sudo mknod "${INITRD_DIR}/dev/null" c 1 3 || exit 255
	sudo mknod "${INITRD_DIR}/dev/tty" c 5 0 || exit 255
	sudo mknod "${INITRD_DIR}/dev/ttyUL0" c 204 187 || exit 255
	sudo mknod "${INITRD_DIR}/dev/ram0" b 1 0 || exit 255
	sudo mknod "${INITRD_DIR}/dev/ram1" b 1 1 || exit 255
	sudo mknod "${INITRD_DIR}/dev/ram2" b 1 2 || exit 255
	sudo mknod "${INITRD_DIR}/dev/ram3" b 1 3 || exit 255
	sudo mknod "${INITRD_DIR}/dev/ram4" b 1 4 || exit 255
	sudo mknod "${INITRD_DIR}/dev/ram5" b 1 5 || exit 255
	sudo mknod "${INITRD_DIR}/dev/ram6" b 1 6 || exit 255
	sudo mknod "${INITRD_DIR}/dev/ram7" b 1 7 || exit 255
	sudo mknod "${INITRD_DIR}/dev/ram8" b 1 8 || exit 255
	sudo mknod "${INITRD_DIR}/dev/ram9" b 1 9 || exit 255
	sudo mknod "${INITRD_DIR}/dev/ram10" b 1 10 || exit 255
	sudo mknod "${INITRD_DIR}/dev/ram11" b 1 11 || exit 255
	sudo mknod "${INITRD_DIR}/dev/ram12" b 1 12 || exit 255
	sudo mknod "${INITRD_DIR}/dev/ram13" b 1 13 || exit 255
	sudo mknod "${INITRD_DIR}/dev/ram14" b 1 14 || exit 255
	sudo mknod "${INITRD_DIR}/dev/ram15" b 1 15 || exit 255

	if [ -n "${EXTRA_DIR}" ]; then
		echo "Installing extra files into initial ramdisk"

		for EXTRA_FILE in ${EXTRA_FILES}; do
			echo "${EXTRA_DIR}/${EXTRA_FILE}"
			sudo mkdir -p "${INITRD_DIR}/opt/"$(dirname "${EXTRA_FILE}")

			EXTRA_FILE_HEAD=$(head -c 4 "${EXTRA_DIR}/${EXTRA_FILE}")
			EXTRA_ELF_HEAD=$(echo -e "\0177ELF")
			if [ "${EXTRA_FILE_HEAD}" = "${EXTRA_ELF_HEAD}" ]; then
				sudo ${POWERPC_CROSS_COMPILER_PREFIX}/bin/powerpc-${CPU}-linux-gnu-strip \
					--strip-unneeded -o "${INITRD_DIR}/opt/${EXTRA_FILE}" \
					"${EXTRA_DIR}/${EXTRA_FILE}" || exit 255
				sudo chmod +x "${INITRD_DIR}/opt/${EXTRA_FILE}" || exit 255
			else
				sudo cp -f "${EXTRA_DIR}/${EXTRA_FILE}" "${INITRD_DIR}/opt/${EXTRA_FILE}" || exit 255
			fi
		done
	fi

	echo "Unmounting uncompressed initial ramdisk image (only sudoers can do that)"
	sudo umount "${INITRD_DIR}" || exit 255
	sudo rm -rf "${INITRD_DIR}"
}

case "${CMD}" in
	all)
		MakeInitRD ${INITRD_IMG} 16384 init.sh '' ''

		MakeInitRD initrd_security_sha_small.img 8192 init_security_sha_small.sh "${MIBENCH_BUILD_DIR}" \
		"security/sha/runme_small.sh \
		security/sha/sha \
		security/sha/input_small.asc \
		"

		MakeInitRD initrd_security_sha_large.img 16384 init_security_sha_large.sh "${MIBENCH_BUILD_DIR}" \
		"security/sha/sha \
		security/sha/runme_large.sh \
		security/sha/input_large.asc \
		"

		MakeInitRD initrd_security_blowfish_small.img 8192 init_security_blowfish_small.sh "${MIBENCH_BUILD_DIR}" \
		"security/blowfish/runme_small.sh \
		security/blowfish/bf \
		security/blowfish/input_small.asc \
		"

		MakeInitRD initrd_security_blowfish_large.img 16384 init_security_blowfish_large.sh "${MIBENCH_BUILD_DIR}" \
		"security/blowfish/bf \
		security/blowfish/runme_large.sh \
		security/blowfish/input_large.asc \
		"

		MakeInitRD initrd_security_rijndael_small.img 8192 init_security_rijndael_small.sh "${MIBENCH_BUILD_DIR}" \
		"security/rijndael/runme_small.sh \
		security/rijndael/rijndael \
		security/rijndael/input_small.asc \
		"

		MakeInitRD initrd_security_rijndael_large.img 16384 init_security_rijndael_large.sh "${MIBENCH_BUILD_DIR}" \
		"security/rijndael/rijndael \
		security/rijndael/runme_large.sh \
		security/rijndael/input_large.asc \
		"

		MakeInitRD initrd_security_pgp.img 8192 init_security_pgp.sh "${MIBENCH_BUILD_DIR}" \
		"security/pgp/runme.sh \
		security/pgp/src/pgp \
		security/pgp/tests/config.txt \
		security/pgp/tests/PGPMacBinaryMappings.txt \
		security/pgp/tests/pubring.bak \
		security/pgp/tests/randseed.bin \
		security/pgp/tests/secring.pgp \
		security/pgp/tests/testin.txt \
		security/pgp/tests/testout.txt.asc \
		security/pgp/tests/PGPgroup.pgr \
		security/pgp/tests/PGPsdkPreferences \
		security/pgp/tests/pubring.pgp \
		security/pgp/tests/randseed.rnd \
		security/pgp/tests/secring.bak \
		security/pgp/tests/secring.skr \
		"

		MakeInitRD initrd_automotive_qsort_small.img 8192 init_automotive_qsort_small.sh "${MIBENCH_BUILD_DIR}" \
		"automotive/qsort/runme_small.sh \
		automotive/qsort/qsort_small \
		automotive/qsort/input_small.dat \
		"

		MakeInitRD initrd_automotive_qsort_large.img 12288 init_automotive_qsort_large.sh "${MIBENCH_BUILD_DIR}" \
		"automotive/qsort/runme_large.sh \
		automotive/qsort/qsort_large \
		automotive/qsort/input_large.dat \
		"

		MakeInitRD initrd_automotive_susan_small.img 8192 init_automotive_susan_small.sh "${MIBENCH_BUILD_DIR}" \
		"automotive/susan/runme_small.sh \
		automotive/susan/susan \
		automotive/susan/input_small.pgm \
		"

		MakeInitRD initrd_automotive_susan_large.img 8192 init_automotive_susan_large.sh "${MIBENCH_BUILD_DIR}" \
		"automotive/susan/susan \
		automotive/susan/runme_large.sh \
		automotive/susan/input_large.pgm \
		"

		MakeInitRD initrd_automotive_basicmath_small.img 8192 init_automotive_basicmath_small.sh "${MIBENCH_BUILD_DIR}" \
		"automotive/basicmath/runme_small.sh \
		automotive/basicmath/basicmath_small \
		"

		MakeInitRD initrd_automotive_basicmath_large.img 24576 init_automotive_basicmath_large.sh "${MIBENCH_BUILD_DIR}" \
		"automotive/basicmath/runme_large.sh \
		automotive/basicmath/basicmath_large \
		"

		MakeInitRD initrd_automotive_bitcount_small.img 8192 init_automotive_bitcount_small.sh "${MIBENCH_BUILD_DIR}" \
		"automotive/bitcount/runme_small.sh \
		automotive/bitcount/bitcnts \
		"

		MakeInitRD initrd_automotive_bitcount_large.img 12288 init_automotive_bitcount_large.sh "${MIBENCH_BUILD_DIR}" \
		"automotive/bitcount/bitcnts \
		automotive/bitcount/runme_large.sh \
		"

		MakeInitRD initrd_network_dijkstra_small.img 8192 init_network_dijkstra_small.sh "${MIBENCH_BUILD_DIR}" \
		"network/dijkstra/runme_small.sh \
		network/dijkstra/dijkstra_small \
		network/dijkstra/input.dat \
		"

		MakeInitRD initrd_network_dijkstra_large.img 8192 init_network_dijkstra_large.sh "${MIBENCH_BUILD_DIR}" \
		"network/dijkstra/input.dat \
		network/dijkstra/runme_large.sh \
		network/dijkstra/dijkstra_large \
		"

		MakeInitRD initrd_network_patricia_small.img 8192 init_network_patricia_small.sh "${MIBENCH_BUILD_DIR}" \
		"network/patricia/runme_small.sh \
		network/patricia/patricia \
		network/patricia/small.udp \
		"

		MakeInitRD initrd_network_patricia_large.img 12288 init_network_patricia_large.sh "${MIBENCH_BUILD_DIR}" \
		"network/patricia/patricia \
		network/patricia/runme_large.sh \
		network/patricia/large.udp \
		"

		MakeInitRD initrd_telecomm_fft_small.img 8192 init_telecomm_fft_small.sh "${MIBENCH_BUILD_DIR}" \
		"telecomm/FFT/runme_small.sh \
		telecomm/FFT/fft \
		"

		MakeInitRD initrd_telecomm_fft_large.img 8192 init_telecomm_fft_large.sh "${MIBENCH_BUILD_DIR}" \
		"telecomm/FFT/fft \
		telecomm/FFT/runme_large.sh \
		"

		MakeInitRD initrd_telecomm_gsm_small.img 8192 init_telecomm_gsm_small.sh "${MIBENCH_BUILD_DIR}" \
		"telecomm/gsm/runme_small.sh \
		telecomm/gsm/bin/toast \
		telecomm/gsm/bin/untoast \
		telecomm/gsm/data/small.au \
		telecomm/gsm/data/small.au.run.gsm \
		"

		MakeInitRD initrd_telecomm_gsm_large.img 12288 init_telecomm_gsm_large.sh "${MIBENCH_BUILD_DIR}" \
		"telecomm/gsm/bin/toast \
		telecomm/gsm/bin/untoast \
		telecomm/gsm/runme_large.sh \
		telecomm/gsm/data/large.au \
		telecomm/gsm/data/large.au.run.gsm \
		"

		MakeInitRD initrd_telecomm_crc32_small.img 8192 init_telecomm_crc32_small.sh "${MIBENCH_BUILD_DIR}" \
		"telecomm/CRC32/runme_small.sh \
		telecomm/CRC32/crc \
		telecomm/adpcm/data/small.pcm \
		"

		MakeInitRD initrd_telecomm_crc32_large.img 40960 init_telecomm_crc32_large.sh "${MIBENCH_BUILD_DIR}" \
		"telecomm/CRC32/crc \
		telecomm/CRC32/runme_large.sh \
		telecomm/adpcm/data/large.pcm \
		"

		MakeInitRD initrd_telecomm_adpcm_small.img 12288 init_telecomm_adpcm_small.sh "${MIBENCH_BUILD_DIR}" \
		"telecomm/adpcm/runme_small.sh \
		telecomm/adpcm/bin/rawcaudio \
		telecomm/adpcm/bin/rawdaudio \
		telecomm/adpcm/data/small.pcm \
		telecomm/adpcm/data/small.adpcm \
		"

		MakeInitRD initrd_telecomm_adpcm_large.img 98304 init_telecomm_adpcm_large.sh "${MIBENCH_BUILD_DIR}" \
		"telecomm/adpcm/bin/rawcaudio \
		telecomm/adpcm/bin/rawdaudio \
		telecomm/adpcm/runme_large.sh \
		telecomm/adpcm/data/large.pcm \
		telecomm/adpcm/data/large.adpcm \
		"

		MakeInitRD initrd_consumer_typeset_small.img 12288 init_consumer_typeset_small.sh "${MIBENCH_BUILD_DIR}" \
		"consumer/typeset/runme_small.sh \
		consumer/typeset/small.lout \
		consumer/typeset/lout-3.24/lout \
		consumer/typeset/lout-3.24/include/tbl \
		consumer/typeset/lout-3.24/include/eiffel \
		consumer/typeset/lout-3.24/include/bsf \
		consumer/typeset/lout-3.24/include/modulaf \
		consumer/typeset/lout-3.24/include/pasf \
		consumer/typeset/lout-3.24/include/slides \
		consumer/typeset/lout-3.24/include/bookf \
		consumer/typeset/lout-3.24/include/graphf \
		consumer/typeset/lout-3.24/include/report \
		consumer/typeset/lout-3.24/include/pythonf \
		consumer/typeset/lout-3.24/include/tabf.lpg \
		consumer/typeset/lout-3.24/include/figf \
		consumer/typeset/lout-3.24/include/pod \
		consumer/typeset/lout-3.24/include/diagf.lpg \
		consumer/typeset/lout-3.24/include/figf.lpg \
		consumer/typeset/lout-3.24/include/mydefs \
		consumer/typeset/lout-3.24/include/russian \
		consumer/typeset/lout-3.24/include/cprint \
		consumer/typeset/lout-3.24/include/diagf \
		consumer/typeset/lout-3.24/include/eq \
		consumer/typeset/lout-3.24/include/reportf \
		consumer/typeset/lout-3.24/include/python \
		consumer/typeset/lout-3.24/include/cprintf \
		consumer/typeset/lout-3.24/include/modula \
		consumer/typeset/lout-3.24/include/tblf.lpg \
		consumer/typeset/lout-3.24/include/langdefs \
		consumer/typeset/lout-3.24/include/book \
		consumer/typeset/lout-3.24/include/diagf.etc \
		consumer/typeset/lout-3.24/include/tabf \
		consumer/typeset/lout-3.24/include/perlf \
		consumer/typeset/lout-3.24/include/podf \
		consumer/typeset/lout-3.24/include/perl \
		consumer/typeset/lout-3.24/include/ts \
		consumer/typeset/lout-3.24/include/eqf \
		consumer/typeset/lout-3.24/include/slidesf \
		consumer/typeset/lout-3.24/include/russian.fd \
		consumer/typeset/lout-3.24/include/picture \
		consumer/typeset/lout-3.24/include/doc \
		consumer/typeset/lout-3.24/include/docf \
		consumer/typeset/lout-3.24/include/bsf.lpg \
		consumer/typeset/lout-3.24/include/fig \
		consumer/typeset/lout-3.24/include/tblf \
		consumer/typeset/lout-3.24/include/blue \
		consumer/typeset/lout-3.24/include/README \
		consumer/typeset/lout-3.24/include/eiffelf \
		consumer/typeset/lout-3.24/include/graphf.lpg \
		consumer/typeset/lout-3.24/include/tab \
		consumer/typeset/lout-3.24/include/picturef \
		consumer/typeset/lout-3.24/include/init \
		consumer/typeset/lout-3.24/include/dsf \
		consumer/typeset/lout-3.24/include/pas \
		consumer/typeset/lout-3.24/include/bluef \
		consumer/typeset/lout-3.24/include/latin2 \
		consumer/typeset/lout-3.24/include/graphf.etc \
		consumer/typeset/lout-3.24/include/graph \
		consumer/typeset/lout-3.24/include/diag \
		consumer/typeset/lout-3.24/data/loutrefs.li \
		consumer/typeset/lout-3.24/data/fontdefs.li \
		consumer/typeset/lout-3.24/data/latin2.ld \
		consumer/typeset/lout-3.24/data/loutrefs.ld \
		consumer/typeset/lout-3.24/data/fcvt \
		consumer/typeset/lout-3.24/data/refstyle.ld \
		consumer/typeset/lout-3.24/data/standard.li \
		consumer/typeset/lout-3.24/data/fontdefs.ld \
		consumer/typeset/lout-3.24/data/standard.ld \
		consumer/typeset/lout-3.24/data/README \
		consumer/typeset/lout-3.24/data/refstyle.li \
		consumer/typeset/lout-3.24/data/oldrefs.ld \
		consumer/typeset/lout-3.24/data/fcvt.awk \
		consumer/typeset/lout-3.24/font/He-NrBdO+ \
		consumer/typeset/lout-3.24/font/ZC-Rm+ \
		consumer/typeset/lout-3.24/font/He-NrO+ \
		consumer/typeset/lout-3.24/font/Symbol \
		consumer/typeset/lout-3.24/font/Bk-MdIt \
		consumer/typeset/lout-3.24/font/He-BdO \
		consumer/typeset/lout-3.24/font/Cr-O+ \
		consumer/typeset/lout-3.24/font/Cr-O \
		consumer/typeset/lout-3.24/font/Pa-BdF \
		consumer/typeset/lout-3.24/font/Ti-BdSC+ \
		consumer/typeset/lout-3.24/font/AG-BdO+ \
		consumer/typeset/lout-3.24/font/Ti-RmSC \
		consumer/typeset/lout-3.24/font/He-Bl+ \
		consumer/typeset/lout-3.24/font/He-CnBd \
		consumer/typeset/lout-3.24/font/He-CnBlO \
		consumer/typeset/lout-3.24/font/AG-CnMd \
		consumer/typeset/lout-3.24/font/Bk-Dm+ \
		consumer/typeset/lout-3.24/font/Ti-BdItF \
		consumer/typeset/lout-3.24/font/Ti-Sm+ \
		consumer/typeset/lout-3.24/font/He-NrBd+ \
		consumer/typeset/lout-3.24/font/Ti-It+ \
		consumer/typeset/lout-3.24/font/Bk-BdIt+ \
		consumer/typeset/lout-3.24/font/He-Bl \
		consumer/typeset/lout-3.24/font/He-O+ \
		consumer/typeset/lout-3.24/font/Pa-Bd+ \
		consumer/typeset/lout-3.24/font/He-ExtC \
		consumer/typeset/lout-3.24/font/He-O \
		consumer/typeset/lout-3.24/font/AG-CnDm+ \
		consumer/typeset/lout-3.24/font/Bk-Dm \
		consumer/typeset/lout-3.24/font/ZC-Lt+ \
		consumer/typeset/lout-3.24/font/He-Bd \
		consumer/typeset/lout-3.24/font/He-CnLiO \
		consumer/typeset/lout-3.24/font/Ti-Rm \
		consumer/typeset/lout-3.24/font/He-Cm \
		consumer/typeset/lout-3.24/font/AG-Dm+ \
		consumer/typeset/lout-3.24/font/ZC-Bd+ \
		consumer/typeset/lout-3.24/font/Pa-Rm+ \
		consumer/typeset/lout-3.24/font/AG-CnDm \
		consumer/typeset/lout-3.24/font/He-Lt \
		consumer/typeset/lout-3.24/font/AG-DmO+ \
		consumer/typeset/lout-3.24/font/NCS-It+ \
		consumer/typeset/lout-3.24/font/Bk-MdIt+ \
		consumer/typeset/lout-3.24/font/AG-ExLt \
		consumer/typeset/lout-3.24/font/He-CnO+ \
		consumer/typeset/lout-3.24/font/He-CnBl \
		consumer/typeset/lout-3.24/font/He-Cn+ \
		consumer/typeset/lout-3.24/font/He-CnLiO+ \
		consumer/typeset/lout-3.24/font/NCS-Bd \
		consumer/typeset/lout-3.24/font/ZC-Bd \
		consumer/typeset/lout-3.24/font/Ti-ExBd \
		consumer/typeset/lout-3.24/font/ZC-LtIt+ \
		consumer/typeset/lout-3.24/font/He-CnBdO \
		consumer/typeset/lout-3.24/font/He-CnBdO+ \
		consumer/typeset/lout-3.24/font/NCS-It \
		consumer/typeset/lout-3.24/font/AG-Bk \
		consumer/typeset/lout-3.24/font/He-NrBd \
		consumer/typeset/lout-3.24/font/He-CnBl+ \
		consumer/typeset/lout-3.24/font/Ti-Bd+ \
		consumer/typeset/lout-3.24/font/Pa-Rm \
		consumer/typeset/lout-3.24/font/Ti-Sm \
		consumer/typeset/lout-3.24/font/He-Cm+ \
		consumer/typeset/lout-3.24/font/ZD \
		consumer/typeset/lout-3.24/font/He-CnO \
		consumer/typeset/lout-3.24/font/He-Cn \
		consumer/typeset/lout-3.24/font/Ti-RmSC+ \
		consumer/typeset/lout-3.24/font/He-Nr+ \
		consumer/typeset/lout-3.24/font/Bk-DmIt+ \
		consumer/typeset/lout-3.24/font/AG-Dm \
		consumer/typeset/lout-3.24/font/Pa-BdF+ \
		consumer/typeset/lout-3.24/font/Ti-BdIt+ \
		consumer/typeset/lout-3.24/font/Pa-It \
		consumer/typeset/lout-3.24/font/He-CnLi \
		consumer/typeset/lout-3.24/font/ZC-Dm+ \
		consumer/typeset/lout-3.24/font/Ti-It \
		consumer/typeset/lout-3.24/font/Ti-ItF+ \
		consumer/typeset/lout-3.24/font/AG-BkO \
		consumer/typeset/lout-3.24/font/AG-CnBd+ \
		consumer/typeset/lout-3.24/font/Ti-BdSC \
		consumer/typeset/lout-3.24/font/AG-MdO+ \
		consumer/typeset/lout-3.24/font/ZC-LtIt \
		consumer/typeset/lout-3.24/font/Cr-Bd+ \
		consumer/typeset/lout-3.24/font/NCS-Rm+ \
		consumer/typeset/lout-3.24/font/Pa-BdIt \
		consumer/typeset/lout-3.24/font/ZC-It \
		consumer/typeset/lout-3.24/font/Ti-ExBd+ \
		consumer/typeset/lout-3.24/font/00README \
		consumer/typeset/lout-3.24/font/Cr+ \
		consumer/typeset/lout-3.24/font/Ti-BdItF+ \
		consumer/typeset/lout-3.24/font/He-UlCm \
		consumer/typeset/lout-3.24/font/He \
		consumer/typeset/lout-3.24/font/AG-ExLtO \
		consumer/typeset/lout-3.24/font/He-ExtC+ \
		consumer/typeset/lout-3.24/font/He-CnBlO+ \
		consumer/typeset/lout-3.24/font/AG-BdO \
		consumer/typeset/lout-3.24/font/ZC-Lt \
		consumer/typeset/lout-3.24/font/AG-Bk+ \
		consumer/typeset/lout-3.24/font/Pa-BdItF+ \
		consumer/typeset/lout-3.24/font/ZC-Rm \
		consumer/typeset/lout-3.24/font/AG-Bd \
		consumer/typeset/lout-3.24/font/Ti-SmIt \
		consumer/typeset/lout-3.24/font/Bk-Bd \
		consumer/typeset/lout-3.24/font/AG-CnBk \
		consumer/typeset/lout-3.24/font/AG-DmO \
		consumer/typeset/lout-3.24/font/He-Nr \
		consumer/typeset/lout-3.24/font/AG-Md+ \
		consumer/typeset/lout-3.24/font/AG-ExLt+ \
		consumer/typeset/lout-3.24/font/He-LtO \
		consumer/typeset/lout-3.24/font/Cr-BdO \
		consumer/typeset/lout-3.24/font/Pa-It+ \
		consumer/typeset/lout-3.24/font/NCS-Rm \
		consumer/typeset/lout-3.24/font/AG-MdO \
		consumer/typeset/lout-3.24/font/Bk-Lt \
		consumer/typeset/lout-3.24/font/He-Lt+ \
		consumer/typeset/lout-3.24/font/ZC-MdIt \
		consumer/typeset/lout-3.24/font/Cr-Bd \
		consumer/typeset/lout-3.24/font/Bk-BdIt \
		consumer/typeset/lout-3.24/font/Bk-Bd+ \
		consumer/typeset/lout-3.24/font/ZC-MdIt+ \
		consumer/typeset/lout-3.24/font/Pa-Bd \
		consumer/typeset/lout-3.24/font/ZC-Dm \
		consumer/typeset/lout-3.24/font/Cr-BdO+ \
		consumer/typeset/lout-3.24/font/Pa-SC+ \
		consumer/typeset/lout-3.24/font/Ti-BdIt \
		consumer/typeset/lout-3.24/font/Bk-Md \
		consumer/typeset/lout-3.24/font/He-BlO \
		consumer/typeset/lout-3.24/font/Bk-LtIt+ \
		consumer/typeset/lout-3.24/font/Ti-Bd \
		consumer/typeset/lout-3.24/font/AG-ExLtO+ \
		consumer/typeset/lout-3.24/font/Pa-BdItF \
		consumer/typeset/lout-3.24/font/He-Bd+ \
		consumer/typeset/lout-3.24/font/AG-BkO+ \
		consumer/typeset/lout-3.24/font/ZC-It+ \
		consumer/typeset/lout-3.24/font/He-CnLi+ \
		consumer/typeset/lout-3.24/font/He-UlCm+ \
		consumer/typeset/lout-3.24/font/Bk-LtIt \
		consumer/typeset/lout-3.24/font/Bk-Lt+ \
		consumer/typeset/lout-3.24/font/Ti-SmIt+ \
		consumer/typeset/lout-3.24/font/He-NrO \
		consumer/typeset/lout-3.24/font/He-BdO+ \
		consumer/typeset/lout-3.24/font/Cr \
		consumer/typeset/lout-3.24/font/Bk-Md+ \
		consumer/typeset/lout-3.24/font/He-LtO+ \
		consumer/typeset/lout-3.24/font/Pa-ItF+ \
		consumer/typeset/lout-3.24/font/AG-CnMd+ \
		consumer/typeset/lout-3.24/font/He+ \
		consumer/typeset/lout-3.24/font/AG-CnBk+ \
		consumer/typeset/lout-3.24/font/Pa-ItF \
		consumer/typeset/lout-3.24/font/AG-CnBd \
		consumer/typeset/lout-3.24/font/Ti-Rm+ \
		consumer/typeset/lout-3.24/font/NCS-BdIt \
		consumer/typeset/lout-3.24/font/Bk-DmIt \
		consumer/typeset/lout-3.24/font/He-BlO+ \
		consumer/typeset/lout-3.24/font/AG-Bd+ \
		consumer/typeset/lout-3.24/font/He-NrBdO \
		consumer/typeset/lout-3.24/font/Pa-SC \
		consumer/typeset/lout-3.24/font/NCS-BdIt+ \
		consumer/typeset/lout-3.24/font/Ti-ItF \
		consumer/typeset/lout-3.24/font/AG-Md \
		consumer/typeset/lout-3.24/font/Pa-BdIt+ \
		consumer/typeset/lout-3.24/font/NCS-Bd+ \
		consumer/typeset/lout-3.24/font/He-CnBd+ \
		consumer/typeset/lout-3.24/maps/null.LCM \
		consumer/typeset/lout-3.24/maps/Std.LCM \
		consumer/typeset/lout-3.24/maps/Symb.LCM \
		consumer/typeset/lout-3.24/maps/Latin1.LCM \
		consumer/typeset/lout-3.24/maps/Ding.LCM \
		consumer/typeset/lout-3.24/maps/LtLatin1.LCM \
		consumer/typeset/lout-3.24/maps/00README \
		consumer/typeset/lout-3.24/maps/LtLatin2.LCM \
		consumer/typeset/lout-3.24/maps/koi8r.LCM \
		consumer/typeset/lout-3.24/hyph/finnish.lh \
		consumer/typeset/lout-3.24/hyph/norweg.lh \
		consumer/typeset/lout-3.24/hyph/slovenia.lh \
		consumer/typeset/lout-3.24/hyph/danish.lh \
		consumer/typeset/lout-3.24/hyph/spanish.lh \
		consumer/typeset/lout-3.24/hyph/dutch.lh \
		consumer/typeset/lout-3.24/hyph/english.lp \
		consumer/typeset/lout-3.24/hyph/italian.lh \
		consumer/typeset/lout-3.24/hyph/czech.lh \
		consumer/typeset/lout-3.24/hyph/german.lh \
		consumer/typeset/lout-3.24/hyph/swedish.lh \
		consumer/typeset/lout-3.24/hyph/portugal.lh \
		consumer/typeset/lout-3.24/hyph/polish.lh \
		consumer/typeset/lout-3.24/hyph/README \
		consumer/typeset/lout-3.24/hyph/engluk.lh \
		consumer/typeset/lout-3.24/hyph/russian.lh \
		consumer/typeset/lout-3.24/hyph/english.lh \
		consumer/typeset/lout-3.24/hyph/french.lh \
		"

		MakeInitRD initrd_consumer_typeset_large.img 12288 init_consumer_typeset_large.sh "${MIBENCH_BUILD_DIR}" \
		"consumer/typeset/lout-3.24/lout \
		consumer/typeset/runme_large.sh \
		consumer/typeset/large.lout \
		consumer/typeset/lout-3.24/include/tbl \
		consumer/typeset/lout-3.24/include/eiffel \
		consumer/typeset/lout-3.24/include/bsf \
		consumer/typeset/lout-3.24/include/modulaf \
		consumer/typeset/lout-3.24/include/pasf \
		consumer/typeset/lout-3.24/include/slides \
		consumer/typeset/lout-3.24/include/bookf \
		consumer/typeset/lout-3.24/include/graphf \
		consumer/typeset/lout-3.24/include/report \
		consumer/typeset/lout-3.24/include/pythonf \
		consumer/typeset/lout-3.24/include/tabf.lpg \
		consumer/typeset/lout-3.24/include/figf \
		consumer/typeset/lout-3.24/include/pod \
		consumer/typeset/lout-3.24/include/diagf.lpg \
		consumer/typeset/lout-3.24/include/figf.lpg \
		consumer/typeset/lout-3.24/include/mydefs \
		consumer/typeset/lout-3.24/include/russian \
		consumer/typeset/lout-3.24/include/cprint \
		consumer/typeset/lout-3.24/include/diagf \
		consumer/typeset/lout-3.24/include/eq \
		consumer/typeset/lout-3.24/include/reportf \
		consumer/typeset/lout-3.24/include/python \
		consumer/typeset/lout-3.24/include/cprintf \
		consumer/typeset/lout-3.24/include/modula \
		consumer/typeset/lout-3.24/include/tblf.lpg \
		consumer/typeset/lout-3.24/include/langdefs \
		consumer/typeset/lout-3.24/include/book \
		consumer/typeset/lout-3.24/include/diagf.etc \
		consumer/typeset/lout-3.24/include/tabf \
		consumer/typeset/lout-3.24/include/perlf \
		consumer/typeset/lout-3.24/include/podf \
		consumer/typeset/lout-3.24/include/perl \
		consumer/typeset/lout-3.24/include/ts \
		consumer/typeset/lout-3.24/include/eqf \
		consumer/typeset/lout-3.24/include/slidesf \
		consumer/typeset/lout-3.24/include/russian.fd \
		consumer/typeset/lout-3.24/include/picture \
		consumer/typeset/lout-3.24/include/doc \
		consumer/typeset/lout-3.24/include/docf \
		consumer/typeset/lout-3.24/include/bsf.lpg \
		consumer/typeset/lout-3.24/include/fig \
		consumer/typeset/lout-3.24/include/tblf \
		consumer/typeset/lout-3.24/include/blue \
		consumer/typeset/lout-3.24/include/README \
		consumer/typeset/lout-3.24/include/eiffelf \
		consumer/typeset/lout-3.24/include/graphf.lpg \
		consumer/typeset/lout-3.24/include/tab \
		consumer/typeset/lout-3.24/include/picturef \
		consumer/typeset/lout-3.24/include/init \
		consumer/typeset/lout-3.24/include/dsf \
		consumer/typeset/lout-3.24/include/pas \
		consumer/typeset/lout-3.24/include/bluef \
		consumer/typeset/lout-3.24/include/latin2 \
		consumer/typeset/lout-3.24/include/graphf.etc \
		consumer/typeset/lout-3.24/include/graph \
		consumer/typeset/lout-3.24/include/diag \
		consumer/typeset/lout-3.24/data/loutrefs.li \
		consumer/typeset/lout-3.24/data/fontdefs.li \
		consumer/typeset/lout-3.24/data/latin2.ld \
		consumer/typeset/lout-3.24/data/loutrefs.ld \
		consumer/typeset/lout-3.24/data/fcvt \
		consumer/typeset/lout-3.24/data/refstyle.ld \
		consumer/typeset/lout-3.24/data/standard.li \
		consumer/typeset/lout-3.24/data/fontdefs.ld \
		consumer/typeset/lout-3.24/data/standard.ld \
		consumer/typeset/lout-3.24/data/README \
		consumer/typeset/lout-3.24/data/refstyle.li \
		consumer/typeset/lout-3.24/data/oldrefs.ld \
		consumer/typeset/lout-3.24/data/fcvt.awk \
		consumer/typeset/lout-3.24/font/He-NrBdO+ \
		consumer/typeset/lout-3.24/font/ZC-Rm+ \
		consumer/typeset/lout-3.24/font/He-NrO+ \
		consumer/typeset/lout-3.24/font/Symbol \
		consumer/typeset/lout-3.24/font/Bk-MdIt \
		consumer/typeset/lout-3.24/font/He-BdO \
		consumer/typeset/lout-3.24/font/Cr-O+ \
		consumer/typeset/lout-3.24/font/Cr-O \
		consumer/typeset/lout-3.24/font/Pa-BdF \
		consumer/typeset/lout-3.24/font/Ti-BdSC+ \
		consumer/typeset/lout-3.24/font/AG-BdO+ \
		consumer/typeset/lout-3.24/font/Ti-RmSC \
		consumer/typeset/lout-3.24/font/He-Bl+ \
		consumer/typeset/lout-3.24/font/He-CnBd \
		consumer/typeset/lout-3.24/font/He-CnBlO \
		consumer/typeset/lout-3.24/font/AG-CnMd \
		consumer/typeset/lout-3.24/font/Bk-Dm+ \
		consumer/typeset/lout-3.24/font/Ti-BdItF \
		consumer/typeset/lout-3.24/font/Ti-Sm+ \
		consumer/typeset/lout-3.24/font/He-NrBd+ \
		consumer/typeset/lout-3.24/font/Ti-It+ \
		consumer/typeset/lout-3.24/font/Bk-BdIt+ \
		consumer/typeset/lout-3.24/font/He-Bl \
		consumer/typeset/lout-3.24/font/He-O+ \
		consumer/typeset/lout-3.24/font/Pa-Bd+ \
		consumer/typeset/lout-3.24/font/He-ExtC \
		consumer/typeset/lout-3.24/font/He-O \
		consumer/typeset/lout-3.24/font/AG-CnDm+ \
		consumer/typeset/lout-3.24/font/Bk-Dm \
		consumer/typeset/lout-3.24/font/ZC-Lt+ \
		consumer/typeset/lout-3.24/font/He-Bd \
		consumer/typeset/lout-3.24/font/He-CnLiO \
		consumer/typeset/lout-3.24/font/Ti-Rm \
		consumer/typeset/lout-3.24/font/He-Cm \
		consumer/typeset/lout-3.24/font/AG-Dm+ \
		consumer/typeset/lout-3.24/font/ZC-Bd+ \
		consumer/typeset/lout-3.24/font/Pa-Rm+ \
		consumer/typeset/lout-3.24/font/AG-CnDm \
		consumer/typeset/lout-3.24/font/He-Lt \
		consumer/typeset/lout-3.24/font/AG-DmO+ \
		consumer/typeset/lout-3.24/font/NCS-It+ \
		consumer/typeset/lout-3.24/font/Bk-MdIt+ \
		consumer/typeset/lout-3.24/font/AG-ExLt \
		consumer/typeset/lout-3.24/font/He-CnO+ \
		consumer/typeset/lout-3.24/font/He-CnBl \
		consumer/typeset/lout-3.24/font/He-Cn+ \
		consumer/typeset/lout-3.24/font/He-CnLiO+ \
		consumer/typeset/lout-3.24/font/NCS-Bd \
		consumer/typeset/lout-3.24/font/ZC-Bd \
		consumer/typeset/lout-3.24/font/Ti-ExBd \
		consumer/typeset/lout-3.24/font/ZC-LtIt+ \
		consumer/typeset/lout-3.24/font/He-CnBdO \
		consumer/typeset/lout-3.24/font/He-CnBdO+ \
		consumer/typeset/lout-3.24/font/NCS-It \
		consumer/typeset/lout-3.24/font/AG-Bk \
		consumer/typeset/lout-3.24/font/He-NrBd \
		consumer/typeset/lout-3.24/font/He-CnBl+ \
		consumer/typeset/lout-3.24/font/Ti-Bd+ \
		consumer/typeset/lout-3.24/font/Pa-Rm \
		consumer/typeset/lout-3.24/font/Ti-Sm \
		consumer/typeset/lout-3.24/font/He-Cm+ \
		consumer/typeset/lout-3.24/font/ZD \
		consumer/typeset/lout-3.24/font/He-CnO \
		consumer/typeset/lout-3.24/font/He-Cn \
		consumer/typeset/lout-3.24/font/Ti-RmSC+ \
		consumer/typeset/lout-3.24/font/He-Nr+ \
		consumer/typeset/lout-3.24/font/Bk-DmIt+ \
		consumer/typeset/lout-3.24/font/AG-Dm \
		consumer/typeset/lout-3.24/font/Pa-BdF+ \
		consumer/typeset/lout-3.24/font/Ti-BdIt+ \
		consumer/typeset/lout-3.24/font/Pa-It \
		consumer/typeset/lout-3.24/font/He-CnLi \
		consumer/typeset/lout-3.24/font/ZC-Dm+ \
		consumer/typeset/lout-3.24/font/Ti-It \
		consumer/typeset/lout-3.24/font/Ti-ItF+ \
		consumer/typeset/lout-3.24/font/AG-BkO \
		consumer/typeset/lout-3.24/font/AG-CnBd+ \
		consumer/typeset/lout-3.24/font/Ti-BdSC \
		consumer/typeset/lout-3.24/font/AG-MdO+ \
		consumer/typeset/lout-3.24/font/ZC-LtIt \
		consumer/typeset/lout-3.24/font/Cr-Bd+ \
		consumer/typeset/lout-3.24/font/NCS-Rm+ \
		consumer/typeset/lout-3.24/font/Pa-BdIt \
		consumer/typeset/lout-3.24/font/ZC-It \
		consumer/typeset/lout-3.24/font/Ti-ExBd+ \
		consumer/typeset/lout-3.24/font/00README \
		consumer/typeset/lout-3.24/font/Cr+ \
		consumer/typeset/lout-3.24/font/Ti-BdItF+ \
		consumer/typeset/lout-3.24/font/He-UlCm \
		consumer/typeset/lout-3.24/font/He \
		consumer/typeset/lout-3.24/font/AG-ExLtO \
		consumer/typeset/lout-3.24/font/He-ExtC+ \
		consumer/typeset/lout-3.24/font/He-CnBlO+ \
		consumer/typeset/lout-3.24/font/AG-BdO \
		consumer/typeset/lout-3.24/font/ZC-Lt \
		consumer/typeset/lout-3.24/font/AG-Bk+ \
		consumer/typeset/lout-3.24/font/Pa-BdItF+ \
		consumer/typeset/lout-3.24/font/ZC-Rm \
		consumer/typeset/lout-3.24/font/AG-Bd \
		consumer/typeset/lout-3.24/font/Ti-SmIt \
		consumer/typeset/lout-3.24/font/Bk-Bd \
		consumer/typeset/lout-3.24/font/AG-CnBk \
		consumer/typeset/lout-3.24/font/AG-DmO \
		consumer/typeset/lout-3.24/font/He-Nr \
		consumer/typeset/lout-3.24/font/AG-Md+ \
		consumer/typeset/lout-3.24/font/AG-ExLt+ \
		consumer/typeset/lout-3.24/font/He-LtO \
		consumer/typeset/lout-3.24/font/Cr-BdO \
		consumer/typeset/lout-3.24/font/Pa-It+ \
		consumer/typeset/lout-3.24/font/NCS-Rm \
		consumer/typeset/lout-3.24/font/AG-MdO \
		consumer/typeset/lout-3.24/font/Bk-Lt \
		consumer/typeset/lout-3.24/font/He-Lt+ \
		consumer/typeset/lout-3.24/font/ZC-MdIt \
		consumer/typeset/lout-3.24/font/Cr-Bd \
		consumer/typeset/lout-3.24/font/Bk-BdIt \
		consumer/typeset/lout-3.24/font/Bk-Bd+ \
		consumer/typeset/lout-3.24/font/ZC-MdIt+ \
		consumer/typeset/lout-3.24/font/Pa-Bd \
		consumer/typeset/lout-3.24/font/ZC-Dm \
		consumer/typeset/lout-3.24/font/Cr-BdO+ \
		consumer/typeset/lout-3.24/font/Pa-SC+ \
		consumer/typeset/lout-3.24/font/Ti-BdIt \
		consumer/typeset/lout-3.24/font/Bk-Md \
		consumer/typeset/lout-3.24/font/He-BlO \
		consumer/typeset/lout-3.24/font/Bk-LtIt+ \
		consumer/typeset/lout-3.24/font/Ti-Bd \
		consumer/typeset/lout-3.24/font/AG-ExLtO+ \
		consumer/typeset/lout-3.24/font/Pa-BdItF \
		consumer/typeset/lout-3.24/font/He-Bd+ \
		consumer/typeset/lout-3.24/font/AG-BkO+ \
		consumer/typeset/lout-3.24/font/ZC-It+ \
		consumer/typeset/lout-3.24/font/He-CnLi+ \
		consumer/typeset/lout-3.24/font/He-UlCm+ \
		consumer/typeset/lout-3.24/font/Bk-LtIt \
		consumer/typeset/lout-3.24/font/Bk-Lt+ \
		consumer/typeset/lout-3.24/font/Ti-SmIt+ \
		consumer/typeset/lout-3.24/font/He-NrO \
		consumer/typeset/lout-3.24/font/He-BdO+ \
		consumer/typeset/lout-3.24/font/Cr \
		consumer/typeset/lout-3.24/font/Bk-Md+ \
		consumer/typeset/lout-3.24/font/He-LtO+ \
		consumer/typeset/lout-3.24/font/Pa-ItF+ \
		consumer/typeset/lout-3.24/font/AG-CnMd+ \
		consumer/typeset/lout-3.24/font/He+ \
		consumer/typeset/lout-3.24/font/AG-CnBk+ \
		consumer/typeset/lout-3.24/font/Pa-ItF \
		consumer/typeset/lout-3.24/font/AG-CnBd \
		consumer/typeset/lout-3.24/font/Ti-Rm+ \
		consumer/typeset/lout-3.24/font/NCS-BdIt \
		consumer/typeset/lout-3.24/font/Bk-DmIt \
		consumer/typeset/lout-3.24/font/He-BlO+ \
		consumer/typeset/lout-3.24/font/AG-Bd+ \
		consumer/typeset/lout-3.24/font/He-NrBdO \
		consumer/typeset/lout-3.24/font/Pa-SC \
		consumer/typeset/lout-3.24/font/NCS-BdIt+ \
		consumer/typeset/lout-3.24/font/Ti-ItF \
		consumer/typeset/lout-3.24/font/AG-Md \
		consumer/typeset/lout-3.24/font/Pa-BdIt+ \
		consumer/typeset/lout-3.24/font/NCS-Bd+ \
		consumer/typeset/lout-3.24/font/He-CnBd+ \
		consumer/typeset/lout-3.24/maps/null.LCM \
		consumer/typeset/lout-3.24/maps/Std.LCM \
		consumer/typeset/lout-3.24/maps/Symb.LCM \
		consumer/typeset/lout-3.24/maps/Latin1.LCM \
		consumer/typeset/lout-3.24/maps/Ding.LCM \
		consumer/typeset/lout-3.24/maps/LtLatin1.LCM \
		consumer/typeset/lout-3.24/maps/00README \
		consumer/typeset/lout-3.24/maps/LtLatin2.LCM \
		consumer/typeset/lout-3.24/maps/koi8r.LCM \
		consumer/typeset/lout-3.24/hyph/finnish.lh \
		consumer/typeset/lout-3.24/hyph/norweg.lh \
		consumer/typeset/lout-3.24/hyph/slovenia.lh \
		consumer/typeset/lout-3.24/hyph/danish.lh \
		consumer/typeset/lout-3.24/hyph/spanish.lh \
		consumer/typeset/lout-3.24/hyph/dutch.lh \
		consumer/typeset/lout-3.24/hyph/english.lp \
		consumer/typeset/lout-3.24/hyph/italian.lh \
		consumer/typeset/lout-3.24/hyph/czech.lh \
		consumer/typeset/lout-3.24/hyph/german.lh \
		consumer/typeset/lout-3.24/hyph/swedish.lh \
		consumer/typeset/lout-3.24/hyph/portugal.lh \
		consumer/typeset/lout-3.24/hyph/polish.lh \
		consumer/typeset/lout-3.24/hyph/README \
		consumer/typeset/lout-3.24/hyph/engluk.lh \
		consumer/typeset/lout-3.24/hyph/russian.lh \
		consumer/typeset/lout-3.24/hyph/english.lh \
		consumer/typeset/lout-3.24/hyph/french.lh \
		"

		MakeInitRD initrd_consumer_lame_small.img 8192 init_consumer_lame_small.sh "${MIBENCH_BUILD_DIR}" \
		"consumer/lame/runme_small.sh \
		consumer/lame/lame3.70/lame \
		consumer/lame/small.wav \
		"

		MakeInitRD initrd_consumer_lame_large.img 12288 init_consumer_lame_large.sh "${MIBENCH_BUILD_DIR}" \
		"consumer/lame/lame3.70/lame \
		consumer/lame/runme_large.sh \
		consumer/lame/large.wav \
		"

		MakeInitRD initrd_consumer_jpeg_small.img 8192 init_consumer_jpeg_small.sh "${MIBENCH_BUILD_DIR}" \
		"consumer/jpeg/runme_small.sh \
		consumer/jpeg/jpeg-6a/cjpeg \
		consumer/jpeg/jpeg-6a/djpeg \
		consumer/jpeg/input_small.ppm \
		consumer/jpeg/input_small.jpg \
		"

		MakeInitRD initrd_consumer_jpeg_large.img 8192 init_consumer_jpeg_large.sh "${MIBENCH_BUILD_DIR}" \
		"consumer/jpeg/jpeg-6a/cjpeg \
		consumer/jpeg/jpeg-6a/djpeg \
		consumer/jpeg/runme_large.sh \
		consumer/jpeg/input_large.ppm \
		consumer/jpeg/input_large.jpg \
		"

		MakeInitRD initrd_office_stringsearch_small.img 8192 init_office_stringsearch_small.sh "${MIBENCH_BUILD_DIR}" \
		"office/stringsearch/runme_small.sh \
		office/stringsearch/search_small \
		"

		MakeInitRD initrd_office_stringsearch_large.img 8192 init_office_stringsearch_large.sh "${MIBENCH_BUILD_DIR}" \
		"office/stringsearch/runme_large.sh \
		office/stringsearch/search_large \
		"
		;;
	clean)
		echo "Cleaning initial ramdisks. Please wait..."
		rm -f ${INITRD_IMG}
		rm -f initrd_security_sha_small.img
		rm -f initrd_security_sha_large.img
		rm -f initrd_security_blowfish_small.img
		rm -f initrd_security_blowfish_large.img
		rm -f initrd_security_rijndael_small.img
		rm -f initrd_security_rijndael_large.img
		rm -f initrd_security_pgp.img
		rm -f initrd_automotive_qsort_small.img
		rm -f initrd_automotive_qsort_large.img
		rm -f initrd_automotive_susan_small.img
		rm -f initrd_automotive_susan_large.img
		rm -f initrd_automotive_basicmath_small.img
		rm -f initrd_automotive_basicmath_large.img
		rm -f initrd_automotive_bitcount_small.img
		rm -f initrd_automotive_bitcount_large.img
		rm -f initrd_network_dijkstra_small.img
		rm -f initrd_network_dijkstra_large.img
		rm -f initrd_network_patricia_small.img
		rm -f initrd_network_patricia_large.img
		rm -f initrd_telecomm_fft_small.img
		rm -f initrd_telecomm_fft_large.img
		rm -f initrd_telecomm_gsm_small.img
		rm -f initrd_telecomm_gsm_large.img
		rm -f initrd_telecomm_crc32_small.img
		rm -f initrd_telecomm_crc32_large.img
		rm -f initrd_telecomm_adpcm_small.img
		rm -f initrd_telecomm_adpcm_large.img
		rm -f initrd_consumer_typeset_small.img
		rm -f initrd_consumer_typeset_large.img
		rm -f initrd_consumer_lame_small.img
		rm -f initrd_consumer_lame_large.img
		rm -f initrd_consumer_jpeg_small.img
		rm -f initrd_consumer_jpeg_large.img
		rm -f initrd_office_stringsearch_small.img
		rm -f initrd_office_stringsearch_large.img
		;;
esac

case "${CMD}" in
	all)
		echo "Your boot ROM image is ready: ${BOOT_ROM}"
		echo "Your Linux kernel is ready: ${VMLINUX}"
		echo "Your initial uncompressed ramdisk images are ready: ${INITRD_IMG} initrd_security_sha_small.img initrd_security_sha_large.img initrd_security_blowfish_small.img initrd_security_blowfish_large.img initrd_security_rijndael_small.img initrd_security_rijndael_large.img initrd_security_pgp.img initrd_automotive_qsort_small.img initrd_automotive_qsort_large.img initrd_automotive_susan_small.img initrd_automotive_susan_large.img initrd_automotive_basicmath_small.img initrd_automotive_basicmath_large.img initrd_automotive_bitcount_small.img initrd_automotive_bitcount_large.img initrd_network_dijkstra_small.img initrd_network_dijkstra_large.img initrd_network_patricia_small.img initrd_network_patricia_large.img initrd_telecomm_fft_small.img initrd_telecomm_fft_large.img initrd_telecomm_gsm_small.img initrd_telecomm_gsm_large.img initrd_telecomm_crc32_small.img initrd_telecomm_crc32_large.img initrd_telecomm_adpcm_small.img initrd_telecomm_adpcm_large.img initrd_consumer_typeset_small.img initrd_consumer_typeset_large.img initrd_consumer_lame_small.img initrd_consumer_lame_large.img initrd_consumer_jpeg_small.img initrd_consumer_jpeg_large.img initrd_office_stringsearch_small.img initrd_office_stringsearch_large.img"
		echo "Your device tree images are ready: ${DEVICE_TREE_BLOB}-6m.dtb ${DEVICE_TREE_BLOB}-8m.dtb ${DEVICE_TREE_BLOB}-12m.dtb ${DEVICE_TREE_BLOB}-16m.dtb ${DEVICE_TREE_BLOB}-24m.dtb ${DEVICE_TREE_BLOB}-32m.dtb  ${DEVICE_TREE_BLOB}-40m.dtb ${DEVICE_TREE_BLOB}-64m.dtb ${DEVICE_TREE_BLOB}-96m.dtb"
		;;
	clean)
		echo "Your build has been cleaned"
		;;
esac
